@@ -2,7 +2,7 @@ package ZeroMQ;
 use 5.008;
 use strict;
 
-our $VERSION = '0.01_03';
+our $VERSION = '0.02_01';
 our @ISA = qw(Exporter);
 
 # TODO: keep in sync with docs below and Makefile.PL
@@ -90,11 +90,21 @@ eval {
     register_write_type(json => \&JSON::encode_json);
 };
 
-
 sub ZeroMQ::Context::socket {
     return ZeroMQ::Socket->new(@_); # $_[0] should contain the context
 }
 
+package
+    ZeroMQ::PollItem::Guard;
+sub new { 
+    my $class = shift;
+    bless { @_ }, $class;
+}
+sub DESTROY {
+    my $self = shift;
+    $self->{pollitem}->remove( $self->{id} );
+}
+
 1;
 __END__
 
@@ -135,19 +145,39 @@ more details on ZeroMQ.
 Loading C<ZeroMQ> will make the L<ZeroMQ::Context>, L<ZeroMQ::Socket>, and 
 L<ZeroMQ::Message> classes available as well.
 
-=head1 FUNCTIONS
+=head1 BASIC USAGE
 
-=head2 device($type, $sock1, $sock2)
+To start using ZeroMQ, you need to create a context object, then as many ZeroMQ::Socket as you need:
 
-=head2 register_read_type($name, \&callback)
+    my $ctxt = ZeroMQ::Context->new;
+    my $socket = $ctxt->socket( ... options );
 
-Register a read callback for a given C<$name>. This is used in C<recv_as()>.
-The callback receives the data received from the socket.
+You need to call C<bind()> or C<connect()> on the socket, depending on your usage. For example on a typical server-client model you would write on the server side:
 
-=head2 register_write_type($name, \&callback)
+    $socket->bind( "tcp://127.0.0.1:9999" );
 
-Register a write callback for a given C<$name>. This is used in C<send_as()>
-The callback receives the Perl structure given to C<send_as()>
+and on the client side:
+
+    $socket->connect( "tcp://127.0.0.1:9999" );
+
+The underlying zeromq library offers TCP, multicast, in-process, and ipc connection patterns. Read the zeromq manual for more details on other ways to setup the socket.
+
+When sending data, you can either pass a ZeroMQ::Message object or a Perl string. 
+
+    # the following two send() calls are equivalent
+    my $msg = ZeroMQ::Message->new( "a simple message" );
+    $socket->send( $msg );
+    $socket->send( "a simple message" ); 
+
+In most cases using ZeroMQ::Message is redundunt, so you will most likely use the string version.
+
+To receive, simply call C<recv()> on the socket
+
+    my $msg = $socket->recv;
+
+The received message is an instance of ZeroMQ::Message object, and you can access the content held in the message via the C<data()> method:
+
+    my $data = $msg->data;
 
 =head1 SERIALIZATION
 
@@ -155,14 +185,15 @@ ZeroMQ.pm comes with a simple serialization/deserialization mechanism.
 
 To serialize, use C<register_write_type()> to register a name and an
 associated callback to serialize the data. For example, for JSON we do
-the following:
+the following (this is already done for you in ZeroMQ.pm if you have
+JSON.pm installed):
 
     use JSON ();
     ZeroMQ::register_write_type('json' => \&JSON::encode_json);
     ZeroMQ::register_read_type('json' => \&JSON::decode_json);
 
-Then you can use C<send_as()> and C<recv_as()> to specify the serialization type as the
-first argument:
+Then you can use C<send_as()> and C<recv_as()> to specify the serialization 
+type as the first argument:
 
     my $ctxt = ZeroMQ::Context->new();
     my $sock = $ctxt->socket( ZMQ_REQ );
@@ -188,6 +219,57 @@ option, do something like this:
 Note that this will have a GLOBAL effect. If you want to change only
 your application, use a name that's different from 'json'.
 
+=head1 ASYNCHRONOUS I/O WITH ZEROMQ
+
+By default ZeroMQ comes with its own poll() mechanism that can handle
+non-blocking sockets. You can use this by creating a ZeroMQ::PollItem
+object:
+
+    my $pi = ZeroMQ::PollItem->new;
+    $pi->add( $zmq_socket, ZMQ_POLLIN, sub {
+        ....
+    } );
+
+Unfortunately this custom polling scheme doesn't play too well with AnyEvent.
+In the near future the zeromq library is believed to come with some API to 
+expose the file descriptor underneath the socket objects -- when that
+happens, we will be able to easily integrate it to AnyEvent. Stay tuned!
+
+=head1 NOTES ON MULTI-PROCESS and MULTI-THREADED USAGE
+
+ZeroMQ works on both multi-process and multi-threaded use cases, but you need
+to be careful bout sharing ZeroMQ objects.
+
+For multi-process environments, you should not be sharing the context object.
+Create separate contexts for each process, and therefore you shouldn't
+be sharing the socket objects either.
+
+For multi-thread environemnts, you can share the same context object. However
+you cannot share sockets.
+
+=head1 FUNCTIONS
+
+=head2 version()
+
+Returns the version of the underlying zeromq library that is being linked.
+In scalar context, returns a dotted version string. In list context,
+returns a 3-element list of the version numbers:
+
+    my $version_string = ZeroMQ::version();
+    my ($major, $minor, $patch) = ZeroMQ::version();
+
+=head2 device($type, $sock1, $sock2)
+
+=head2 register_read_type($name, \&callback)
+
+Register a read callback for a given C<$name>. This is used in C<recv_as()>.
+The callback receives the data received from the socket.
+
+=head2 register_write_type($name, \&callback)
+
+Register a write callback for a given C<$name>. This is used in C<send_as()>
+The callback receives the Perl structure given to C<send_as()>
+
 =head1 EXPORTS
 
 You may choose to import one or more (using the C<:all> import tag)
@@ -196,8 +278,6 @@ C<use ZeroMQ> call as shown in the synopsis above.
 
 The exportable constants are:
 
-=head1 EXPORTS
-
 =head2 C<:socket> - Socket types and socket options
 
 =over 4
@@ -299,13 +379,7 @@ The exportable constants are:
 =head1 CAVEATS
 
 This is an early release. Proceed with caution, please report
-(or better yet: fix) bugs you encounter. Tested againt 0MQ 2.0.7.
-
-Use of the C<inproc://> transport layer doesn't seem to work
-between two perl ithreads. This may be due to the fact that right now,
-context aren't shared between ithreads and C<inproc> works
-only within a single context. Try another transport layer until
-contexts can be shared.
+(or better yet: fix) bugs you encounter. Tested againt 0MQ 2.0.8.
 
 =head1 SEE ALSO
 
@@ -315,15 +389,15 @@ L<http://zeromq.org>
 
 =head1 AUTHOR
 
-Steffen Mueller, C<< <smueller@cpan.org> >>
-
 Daisuke Maki C<< <daisuke@endeworks.jp> >>
 
+Steffen Mueller, C<< <smueller@cpan.org> >>
+
 =head1 COPYRIGHT AND LICENSE
 
 The ZeroMQ module is
 
-Copyright (C) 2010 by Steffen Mueller
+Copyright (C) 2010 by Daisuke Maki
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.0 or,
