@@ -10,9 +10,18 @@ typedef struct {
     void *ctxt;
     unsigned int count;
 } PerlZMQ_Context ;
+
 typedef void PerlZMQ_Socket;
 typedef zmq_msg_t PerlZMQ_Message;
 
+typedef struct {
+    int bucket_size;
+    int item_count;
+    zmq_pollitem_t **items;
+    char **item_ids;
+    SV  **callbacks;
+} PerlZMQ_PollItem;
+
 /* ZMQ_PULL was introduced for version 3, but it exists in git head.
  * it's just rename of ZMQ_UPSTREAM and ZMQ_DOWNSTREAM so we just
  * fake it here
